// Popis: Umozni pozicovani prvku
// Autor: VasaM
// Licence: CC BY 4.0 CZ



// Validace spravnosti souradnic (zamezi vyjeti mimo okno)
function validateX(obj, x) {
	x = x < 0 ? 0 : x;

	if (x + $(obj).outerWidth() > $(document).innerWidth()) {
		x = $(document).innerWidth() - $(obj).outerWidth();
	}

	return x;
}

function validateY(obj, y) {
	y = y < 0 ? 0 : y;

	if (y + $(obj).outerHeight() > $(document).innerHeight()) {
		y = $(document).innerHeight() - $(obj).outerHeight();
	}

	return y;
}

function validate(obj, coords) {
	return {x: validateX(obj, coords.x), y: validateY(obj, coords.y)};
}



// Umozni pohyb prvkem
function movable(obj) {
	let offset = [0, 0];
	let isDown = false;
	let locking = false;

	let corner = $('<div class="movable-corner"></div>');
	$(obj).append(corner);
	
	$(corner)
		.mousedown(function(e) {
			if (! $(obj).data('locked') && !locking) {
				isDown = true;

				offset = [
				    obj.offsetLeft - e.clientX,
				    obj.offsetTop - e.clientY
				];
			}
		})
		.mouseover(function() {
			document.onkeydown = function(e) {
				if (e.keyCode == code('L')) {
					$(obj).addClass('movable-locking');
					locking = true;
				}
			};

			document.onkeyup = function() {
				$(obj).removeClass('movable-locking');
				locking = false;
			};
		})
		.mouseleave(function() {
			document.onkeydown = undefined;
			$(obj).removeClass('movable-locking');
			locking = false;
		})
		.click(function() {
			if (locking) {
				$(obj).toggleClass('movable-locked');
				$(obj).data('locked', ! $(obj).data('locked'));
				saveLock(obj, $(obj).data('locked'));
			}
		});


	document.addEventListener('mouseup', function() {
		isDown = false;
		savePos(obj);
	}, true);

	document.addEventListener('mousemove', function(event) {
		event.preventDefault();
		if (isDown) {
		    let position = validate(obj, {
		        x : event.clientX + offset[0],
		        y : event.clientY + offset[1]
		    });


		    obj.style.left = position.x + 'px';
		    obj.style.top  = position.y + 'px';
		}
	}, true);	
}



// Ulozeni pozice
function savePos(obj) {
	localStorage.setItem('movable_' + $(obj).attr('id') + '_top', obj.offsetTop);
	localStorage.setItem('movable_' + $(obj).attr('id') + '_left', obj.offsetLeft);
}


// Nacteni pozice
function loadPos(obj) {
	let x = localStorage.getItem('movable_' + $(obj).attr('id') + '_left');
	let y = localStorage.getItem('movable_' + $(obj).attr('id') + '_top');

	if (x != undefined && y != undefined) {
		obj.style.left = validateX(obj, parseInt(x, 10)) + 'px';
		obj.style.top  = validateY(obj, parseInt(y, 10)) + 'px';
	}
}


// Ulozeni zamku
function saveLock(obj, lock) {
	localStorage.setItem('movable_' + $(obj).attr('id') + '_lock', lock);
}


// Nacteni pozice
function loadLock(obj) {
	let lock = localStorage.getItem('movable_' + $(obj).attr('id') + '_lock');

	if (lock == null) {
		lock = false;
	}
	else if (lock) {
		$(obj).toggleClass('movable-locked');		
	}

	$(obj).data('locked', lock);
}




function movableAddForBasic() {
	// $('#sidebar').addClass('movable');
	$('#filter-panel').addClass('movable');
	// $('#pointinfo').addClass('movable');
	// $('.leaflet-control-scale').addClass('movable').attr('id', 'mapScale');
	// $('.leaflet-control-geocoder').addClass('movable').attr('id', 'search');
}



// Inicializace
function Movable_ggm_init() {
	'use strict';

	loadcss('plugin/Movable.css');

	ggmregister('onload', function() {
		addSettings('Movable', '<button id="reset-movable">Reset Movable</button>');
		$('#reset-movable').click(function() {
			$.each(localStorage, function(key, val) {
				if (key.substr(0,8) == 'movable_' && (key.substr(-5) == '_left' || key.substr(-4) == '_top' || key.substr(-5) == '_lock')) {
					localStorage.removeItem(key);
				}
			});
		});
	});

	ggmregister('onmap', function() {
		movableAddForBasic();
		let movables = $('.movable');
		
		$.each(movables, function(key, obj) {
			loadPos(obj);
			loadLock(obj);
			movable(obj);
		});
	});
}